﻿using System;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.Controls;
using Telerik.Windows.Documents.Model;
using Telerik.Windows.Documents.FormatProviders;
using Telerik.Windows.Documents.FormatProviders.Html;
using System.IO;
using Telerik.Windows.Documents.Layout;
using System.Windows.Media;

namespace SilverlightApplication1
{
    public partial class RadRichTextBoxEx : UserControl
    {
        public RadRichTextBoxEx()
        {
            InitializeComponent();
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            this.radRichTextBox.Document.LayoutMode = DocumentLayoutMode.Flow;
            this.radRichTextBox.Document.SectionDefaultPageMargin = new Padding(2, 2, 2, 2);
            this.radRichTextBox.Document.ParagraphDefaultSpacingAfter = 0;

            this.radRichTextBox.Document.Measure(RadDocument.MAX_DOCUMENT_SIZE);
            this.radRichTextBox.Document.Arrange(new RectangleF(PointF.Empty, this.radRichTextBox.Document.DesiredSize));

            var defaultSpan = (Span)this.radRichTextBox.Document.Sections.First.Paragraphs.First.Inlines.First;
            defaultSpan.FontFamily = new FontFamily("Verdana");
            defaultSpan.FontSize = 12;
            defaultSpan.ForeColor = Colors.Red;

            this.radRichTextBox.DocumentContentChanged += new EventHandler(radRichTextBox_DocumentContentChanged);
        }

        public static DependencyProperty HtmlProperty = DependencyProperty.Register(
            "Html", typeof(string), typeof(RadRichTextBoxEx), new PropertyMetadata(string.Empty, HtmlChanged));

        private static bool changeInitiatedFromRtb;

        public string Html
        {
            get 
            {
                return (string)this.GetValue(HtmlProperty);
            }
            set 
            {
                this.SetValue(HtmlProperty, value);
            }
        }

        private static void SetupNewDocument(RadRichTextBoxEx control, RadDocument oldDocument, RadDocument newDocument)
        {
            newDocument.LayoutMode = oldDocument.LayoutMode;
            newDocument.DefaultPageLayoutSettings.Height = oldDocument.DefaultPageLayoutSettings.Height;
            newDocument.DefaultPageLayoutSettings.Width = oldDocument.DefaultPageLayoutSettings.Width;
            newDocument.SectionDefaultPageMargin = oldDocument.SectionDefaultPageMargin;
            newDocument.Sections.First.PageMargin = oldDocument.SectionDefaultPageMargin;

            newDocument.ParagraphDefaultSpacingBefore = oldDocument.ParagraphDefaultSpacingBefore;
            newDocument.ParagraphDefaultSpacingAfter = oldDocument.ParagraphDefaultSpacingAfter;
        }

        private static void HtmlChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            if (!changeInitiatedFromRtb)
            {
                var control = ((RadRichTextBoxEx)sender);
                var oldDocument = control.radRichTextBox.Document;
                var newDocument = GetDocumentFromHtml((string)e.NewValue);

                newDocument.LayoutMode = DocumentLayoutMode.Flow;
                control.radRichTextBox.Document = newDocument;
                SetupNewDocument(control, oldDocument, newDocument);

                control.radRichTextBox.UpdateEditorLayout();
            }
        }

        private static string GetHtmlFormDocument(RadDocument document)
        {
            IDocumentFormatProvider exporter = new HtmlFormatProvider();
            string result;

            using (MemoryStream stream = new MemoryStream())
            {
                exporter.Export(document, stream);
                stream.Seek(0, SeekOrigin.Begin);

                using (StreamReader reader = new StreamReader(stream))
                {
                    result = reader.ReadToEnd();
                }
            }

            return result;
        }

        private static RadDocument GetDocumentFromHtml(string html)
        {
            IDocumentFormatProvider provider = new HtmlFormatProvider();
            using (MemoryStream stream = new MemoryStream())
            {
                StreamWriter writer = new StreamWriter(stream);
                writer.Write(html);
                writer.Flush();
                stream.Seek(0, SeekOrigin.Begin);
                return provider.Import(stream);                
            }
        }

        public RadRichTextBox RichTextBox
        {
            get
            {
                return this.radRichTextBox;
            }
        }

        void radRichTextBox_DocumentContentChanged(object sender, EventArgs e)
        {
            changeInitiatedFromRtb = true;
            try
            {
                this.SetValue(HtmlProperty, GetHtmlFormDocument(this.radRichTextBox.Document));
            }
            finally
            {
                changeInitiatedFromRtb = false;
            }
        }
    }
}
